<?php
/**
 * Zen
 *
 * @author    Zen
 * @copyright Copyright (c) 2022 Zen
 * @license   https://opensource.org/licenses/AFL-3.0  Academic Free License ("AFL") v. 3.0
 *
 * https://www.zen.com/
 */

use Zen\Payment\Api;
use Zen\Payment\CartData;
use Zen\Payment\Paywall;
use Zen\Payment\Util;

if (!defined('_PS_VERSION_')) {
    exit;
}

/**
 * Class ZencomPaymentModuleFrontController
 *
 * @property bool display_column_left
 * @property bool display_column_right
 */
class ZencomPaymentModuleFrontController extends ModuleFrontController
{

    /**
     * Initialize controller
     *
     * @see FrontController::init()
     */
    public function init()
    {
        $this->display_column_left = false;
        $this->display_column_right = false;
        parent::init();
    }

    /**
     * @throws NotSupportedHashMethod
     * @throws OrderIsNotAnArray
     */
    public function initContent()
    {
        parent::initContent();

        $cart = $this->context->cart;

        if (!$cart->date_upd) {
            Tools::redirect('/');

            return;
        }

        $zen = new Zencom();

        $cartId = $cart->id;

        $customer = new Customer((int)$cart->id_customer);

        $zen->validateOrder(
            (int)$cartId,
            Configuration::get('PAYMENT_ZEN_NEW_STATUS'),
            $cart->getOrderTotal(),
            $zen->displayName,
            null,
            [],
            null,
            false,
            $customer->secure_key
        );

        $orderId = Order::getOrderByCartId((int)$cartId);
        $order = new Order($orderId);

        $currencyInfo = Currency::getCurrency($order->id_currency);

        $total = Util::convertAmount($order->total_paid);

        $successUrl = $this->context->link->getPageLink('order-confirmation', true, null, [
            'id_cart' => $cart->id,
            'key' => $customer->secure_key,
            'id_module' => $zen->id,
            'id_order' => $orderId,
        ]);

        $api = new Api(
            Configuration::get('ZEN_API_KEY'),
            $this->getEnvironment()
        );

        $paymentData = Paywall::prepareOrderData(
            $total,
            $currencyInfo['iso_code'],
            $orderId,
            $customer->firstname,
            $customer->lastname,
            $customer->email,
            $successUrl,
            $this->context->link->getModuleLink('zencom', 'failure'),
            $successUrl,
            $this->context->link->getModuleLink('zencom', 'notification'),
            $this->getCart($cart, $total),
            Configuration::get('ZEN_TERMINAL_ID'),
            Configuration::get('ZEN_PAYWALL_SECRET'),
            $zen->name,
            $zen->version,
            $zen->platformName,
            _PS_VERSION_
        );

        $paymentRequest = $api->createPayment(json_encode($paymentData));
        $paymentError = !$paymentRequest['success'];

        $this->context->smarty->assign([
            'zen_msg_redirect_hint' => $this->module->l(
                'Please wait, in the next step you will choose a payment method',
                'payment'
            ),
        ]);

        if ($paymentError) {
            $history = new OrderHistory();
            $history->id_order = $orderId;
            $history->changeIdOrderState(_PS_OS_ERROR_, $orderId);
            $history->addWithemail(true);

            $this->setTemplate('error.tpl');
        } else {
            header('Location:' . $paymentRequest['body']['redirectUrl']);
            die();
        }
    }

    /**
     * @return string
     */
    private function getEnvironment()
    {
        return Util::ENVIRONMENT_PRODUCTION;
    }

    /**
     * @param Cart $cart
     * @param int $total
     *
     * @return array
     * @throws Exception
     */
    public function getCart($cart, $total)
    {

        $cartData = new CartData();
        $cartData->setAmount($total);

        foreach ($cart->getProducts() as $product) {
            $productPrice = Util::convertAmount($product['price_wt']);

            $cartData->addItem(
                $product['name'],
                $productPrice,
                $product['quantity'],
                Util::convertAmount($productPrice * $product['quantity'])
            );
        }

        $carrier = new Carrier($cart->id_carrier);

        $shippingCost = Util::convertAmount($cart->getTotalShippingCost());

        $cartData->addItem(
            isset($carrier->name) ? $carrier->name : $this->module->l('Delivery cost'),
            $shippingCost,
            1,
            $shippingCost
        );

        $discount = Util::convertAmount($cart->getOrderTotal(true, Cart::ONLY_DISCOUNTS));

        if ($discount > 0) {
            $cartData->addItem(
                $this->module->l('Discount'),
                -$discount,
                1,
                -$discount
            );
        }

        return $cartData->getCartData($this->module->l('Round'));
    }
}
